use std::sync::Arc;

use axum::{
    extract::State,
    response::{ErrorResponse, Html, Redirect},
    routing::{get, post, IntoMakeService},
    Json, Router,
};
use serde::Serialize;
use tokio::sync::RwLock;

use crate::{
    web::templates::load_templates,
    types::{AppState, PageContext},
    task::{MY_ADDRESS, collect_local_address},
};

async fn root(State(state): State<Arc<RwLock<AppState>>>) -> axum::response::Result<Html<String>> {
    let state = state.read().await;

    let address_guard = MY_ADDRESS.lock().await;
    let address = address_guard.clone();
    drop(address_guard);

    let ctx = match tera::Context::from_serialize(&PageContext {
        address,
    }) {
        Ok(ctx) => ctx,
        Err(err) => return Err(ErrorResponse::from(format!("{err}"))),
    };
    match state.templates.render("index.html", &ctx) {
        Ok(result) => Ok(Html::from(result)),
        Err(err) => Err(ErrorResponse::from(format!("{err}"))),
    }
}

#[derive(Serialize, Default)]
struct ReloadResult {
    #[serde(skip_serializing_if = "Option::is_none")]
    error: Option<String>,
}

async fn reload(State(state): State<Arc<RwLock<AppState>>>) -> Result<Redirect, Json<ReloadResult>> {
    let mut state = state.write_owned().await;
    if let Err(err) = state.templates.full_reload() {
        return Err(Json(ReloadResult {
            error: Some(err.to_string()),
        }));
    }
    drop(state);
    if let Err(err) = collect_local_address().await {
        return Err(Json(ReloadResult {
            error: Some(err.to_string()),
        }));
    }

    Ok(Redirect::to("/"))
}

pub fn make_service() -> Result<IntoMakeService<Router<()>>, Box<dyn std::error::Error>> {
    let templates = load_templates()?;
    
    let router = Router::new();

    #[cfg(feature = "serve-static")]
    let router = router.route_service("/static/*path", staticfiles::StaticFiles::strip("/static/"));

    Ok(router
        .route("/", get(root))
        .route("/reload", post(reload))
        .with_state(Arc::new(RwLock::new(crate::types::AppState {
            templates: templates,
        })))
        .into_make_service())
}
